# Defaults for ecryptfs-utils initscript
# sourced by /etc/init.d/ecryptfs-utils
# installed at /etc/default/ecryptfs-utils by the maintainer scripts

#
# This is a POSIX shell fragment
#

# Additional options that are passed to the Daemon.
DAEMON_OPTS=""
