/**
 * Copyright (C) 2006 International Business Machines Corp.
 * Author(s): Mike Halcrow <mhalcrow@us.ibm.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include "../include/ecryptfs.h"
#include "../include/decision_graph.h"


int set_key_data(unsigned char *passphrase_data, char *sig, int *length)
{
	return -1;
}

int ecryptfs_generate_key(char *filename)
{
	return -1;
}

int ecryptfs_encrypt(int size, char *from, char *to,
		     unsigned char *private_key_data)
{
	return -1;
}

int decrypt(char *from, char *to, size_t *decrypted_key_size,
	    unsigned char *passphrase_data)
{
	return -1;
}

int get_pki_data_length(struct ecryptfs_name_val_pair *pair)
{
	return -1;
}

int set_pki_data(struct ecryptfs_name_val_pair *pair, unsigned char *data)
{
	return -1;
}

static int get_param_subgraph_trans_node(struct transition_node **trans,
					 uint32_t version)
{
	return -1;
}

struct ecryptfs_pki_ops ops = {
	&set_key_data,
	&ecryptfs_generate_key,
	&ecryptfs_encrypt,
	&decrypt,
	&get_pki_data_length,
	&set_pki_data,
	&get_param_subgraph_trans_node
};

int init_pki(char **pki_name, struct ecryptfs_pki_elem *pki)
{
	return -1;
}
